/// <reference types="node" />
import type { Matcher } from 'anymatch';
import type { AwaitWriteFinishOptions, WatchOptions } from 'chokidar';
import type { Writable } from 'stream';
import type { AcceptedPlugin, ProcessOptions, SourceMapOptions } from 'postcss';
import type { IPluginOptions } from '../postcssPlugin/types';
export interface ISessionOptions {
    include?: Array<string> | string;
    exclude?: Matcher;
    extensions?: Array<string>;
    helper?: string;
    ext?: string;
    css?: string;
    watch?: boolean;
    chokidar?: WatchOptions;
    postcssPlugins?: Array<AcceptedPlugin>;
    postcssOptions?: ProcessOptions;
    esifycssPluginParameter?: IPluginOptions;
    stdout?: Writable;
    stderr?: Writable;
}
export interface IReadonlyWatchOptions extends Readonly<WatchOptions> {
    awaitWriteFinish?: Readonly<AwaitWriteFinishOptions> | boolean;
}
export interface ISessionOutput {
    type: 'css' | 'script';
    path: string;
}
export interface ISessionConfiguration {
    readonly watch: boolean;
    readonly output: ISessionOutput;
    readonly ext: string;
    readonly path: ReadonlyArray<string>;
    readonly chokidar: IReadonlyWatchOptions;
    readonly stdout: Writable;
    readonly stderr: Writable;
    readonly postcssPlugins: Array<AcceptedPlugin>;
    readonly postcssOptions: ProcessOptions;
    readonly cssKey: string;
}
export interface ICSSParserParameters {
    file: string;
    css?: Buffer | string;
    options?: ProcessOptions;
    plugins: Array<AcceptedPlugin>;
    map?: SourceMapOptions;
}
export interface ICSSParserConfigurations {
    readonly css: string;
    readonly plugins: Array<AcceptedPlugin>;
    readonly options: {
        readonly from: string;
        readonly map: SourceMapOptions;
    };
}
