"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSandbox = exports.walkElements = exports.isStyleElement = void 0;
class Element {
    constructor(tagName) {
        this.tagName = tagName;
        this.children = [];
    }
    appendChild(element) {
        this.children.push(element);
        return element;
    }
}
class CSSRule {
    constructor(cssText) {
        this.cssText = cssText;
    }
}
class StyleSheet {
    constructor() {
        this.cssRules = [];
    }
    insertRule(cssText, index = 0) {
        this.cssRules.splice(index, 0, new CSSRule(cssText));
    }
}
class StyleElement extends Element {
    constructor() {
        super('style');
        this.sheet = new StyleSheet();
    }
}
const isStyleElement = (element) => element.tagName === 'style';
exports.isStyleElement = isStyleElement;
const walkElements = function* (element) {
    yield element;
    for (const childElement of element.children) {
        yield* exports.walkElements(childElement);
    }
};
exports.walkElements = walkElements;
class Document {
    constructor() {
        this.head = new Element('head');
        this.body = new Element('body');
    }
    createElement(tagName) {
        switch (tagName) {
            case 'style':
                return new StyleElement();
            default:
                return new Element(tagName);
        }
    }
    *walkElements() {
        yield* exports.walkElements(this.head);
        yield* exports.walkElements(this.body);
    }
    get stylesheets() {
        const result = [];
        for (const element of this.walkElements()) {
            if (exports.isStyleElement(element)) {
                result.push(element.sheet);
            }
        }
        return result;
    }
    get css() {
        return this.stylesheets.map((sheet) => sheet.cssRules.map((rule) => rule.cssText).join('')).join('');
    }
}
const createSandbox = () => ({
    document: new Document(),
    exports: {},
});
exports.createSandbox = createSandbox;
