"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCode = void 0;
const path = require("path");
const vm = require("vm");
const rollup = require("rollup");
const postcss = require("postcss");
const createSandbox_for_test_1 = require("../util/createSandbox.for-test");
const writeFilep_1 = require("./writeFilep");
const runCode = async (file) => {
    const testCodePath = `${file}-import.js`;
    await writeFilep_1.writeFilep(testCodePath, `import * as imported from './${path.basename(file)}';exports = imported;`);
    const bundle = await rollup.rollup({ input: testCodePath });
    const { output: [output, undef] } = await bundle.generate({ format: 'es' });
    if (undef) {
        if ('code' in undef) {
            throw new Error(`Unexpected multiple outputs: ${undef.code}`);
        }
        else {
            throw new Error(`Unexpected multiple outputs: ${JSON.stringify(undef, null, 2)}`);
        }
    }
    const sandbox = createSandbox_for_test_1.createSandbox();
    vm.runInNewContext(output.code, sandbox);
    const { exports: { className = {}, id = {}, keyframes = {} } } = sandbox;
    return { className, id, keyframes, root: postcss.parse(sandbox.document.css) };
};
exports.runCode = runCode;
